@@ -955,6 +955,8 @@ readok2:
 
 int uwsgi_proxy_nb(struct wsgi_request *wsgi_req, char *addr, struct uwsgi_buffer *ub, size_t remains, int timeout) {
 
+	struct uwsgi_buffer *headers = NULL;
+
         int fd = uwsgi_connect(addr, 0, 1);
         if (fd < 0) {
 		return -1;
@@ -988,21 +990,91 @@ int uwsgi_proxy_nb(struct wsgi_request *wsgi_req, char *addr, struct uwsgi_buffe
         }
 
         // read the response
+	headers = uwsgi_buffer_new(8192);
+	// max 64k headers
+	ub->limit = UMAX16;
         for(;;) {
                 char buf[8192];
                 ssize_t rlen = uwsgi_read_true_nb(fd, buf, 8192, timeout);
                 if (rlen > 0) {
-                        if (uwsgi_response_write_body_do(wsgi_req, buf, rlen)) {
-                                break;
-                        }
+			if (headers) {
+				if (uwsgi_buffer_append(headers, buf, rlen)) {
+					goto end;
+				}
+				// check if we have a full HTTP response
+				if (uwsgi_is_full_http(headers)) {
+					int ret = uwsgi_blob_to_response(wsgi_req, headers->buf, headers->pos);	
+					if (ret) continue;
+					uwsgi_buffer_destroy(headers);
+					headers = NULL;
+				}
+			}
+			else {
+                        	if (uwsgi_response_write_body_do(wsgi_req, buf, rlen)) {
+                                	break;
+                        	}
+			}
                         continue;
                 }
                 break;
         }
+	if (headers) uwsgi_buffer_destroy(headers);
 
 	close(fd);
 	return 0;
 end:
+	if (headers) uwsgi_buffer_destroy(headers);
 	close(fd);
 	return -1;
 }
+
+void uwsgi_file_write_do(struct uwsgi_string_list *usl) {
+
+	struct uwsgi_string_list *fl = usl;
+	while(fl) {
+		char *equal = strchr(fl->value, '=');
+		if (equal) {
+			*equal = 0;
+			FILE *f = fopen(fl->value, "w");
+			if (!f) {
+				uwsgi_error_open("uwsgi_file_write_do()");
+				exit(1);
+			}
+			uwsgi_log("writing \"%s\" to \"%s\" ...\n", equal+1, fl->value);
+			if (fprintf(f, "%s\n", equal+1) <= 0 || ferror(f) || fclose(f)) {
+				uwsgi_error("uwsgi_file_write_do()");
+				exit(1);
+			}
+		}
+		else {
+			uwsgi_log("unable to write empty value for \"%s\"\n", fl->value);
+			exit(1);
+		}
+		*equal = '=';	
+		fl = fl->next;
+	}
+}
+
+int uwsgi_fd_is_safe(int fd) {
+	int i;
+	for(i=0;i<uwsgi.safe_fds_cnt;i++) {
+		if (uwsgi.safe_fds[i] == fd) {
+			return 1;
+		}
+	}
+	return 0;
+}
+void uwsgi_add_safe_fd(int fd) {
+	// check if the fd is already safe
+	if (uwsgi_fd_is_safe(fd)) return;
+
+	size_t len = sizeof(int) * (uwsgi.safe_fds_cnt+1);
+	int *tmp = realloc(uwsgi.safe_fds, len);
+	if (!tmp) {
+		uwsgi_error("uwsgi_add_safe_fd()/realloc()");
+		exit(1);
+	}
+	uwsgi.safe_fds = tmp;	
+	uwsgi.safe_fds[uwsgi.safe_fds_cnt] = fd;	
+	uwsgi.safe_fds_cnt++;
+}
