@@ -351,6 +351,9 @@ void uwsgi_reload(char **argv) {
 	/* check fd table (a module can obviosly open some fd on initialization...) */
 	uwsgi_log("closing all non-uwsgi socket fds > 2 (max_fd = %d)...\n", (int) uwsgi.max_fd);
 	for (i = 3; i < (int) uwsgi.max_fd; i++) {
+
+		if (uwsgi_fd_is_safe(i)) continue;
+
 		int found = 0;
 
 		struct uwsgi_socket *uwsgi_sock = uwsgi.sockets;
@@ -413,6 +416,13 @@ void uwsgi_reload(char **argv) {
 #endif
 	}
 
+#ifndef UWSGI_IPCSEM_ATEXIT
+	// free ipc semaphores if in use
+	if (!strcmp(uwsgi.lock_engine, "ipcsem")) {
+		uwsgi_ipcsem_clear();
+	}
+#endif
+
 	uwsgi_log("running %s\n", uwsgi.binary_path);
 	uwsgi_flush_logs();
 	argv[0] = uwsgi.binary_path;
@@ -630,178 +640,6 @@ int uwsgi_respawn_worker(int wid) {
 	return 0;
 }
 
-
-void uwsgi_manage_signal_cron(time_t now) {
-
-	struct tm *uwsgi_cron_delta;
-	int i;
-
-	uwsgi_cron_delta = localtime(&now);
-
-	if (uwsgi_cron_delta) {
-
-		// fix month
-		uwsgi_cron_delta->tm_mon++;
-
-		uwsgi_lock(uwsgi.cron_table_lock);
-		for (i = 0; i < ushared->cron_cnt; i++) {
-
-			struct uwsgi_cron *ucron = &ushared->cron[i];
-			int uc_minute, uc_hour, uc_day, uc_month, uc_week;
-
-			uc_minute = ucron->minute;
-			uc_hour = ucron->hour;
-			uc_day = ucron->day;
-			uc_month = ucron->month;
-			uc_week = ucron->week;
-
-			// negative values as interval -1 = * , -5 = */5
-			if (ucron->minute < 0) {
-				if ((uwsgi_cron_delta->tm_min % abs(ucron->minute)) == 0) {
-					uc_minute = uwsgi_cron_delta->tm_min;
-				}
-			}
-			if (ucron->hour < 0) {
-				if ((uwsgi_cron_delta->tm_hour % abs(ucron->hour)) == 0) {
-					uc_hour = uwsgi_cron_delta->tm_hour;
-				}
-			}
-			if (ucron->month < 0) {
-				if ((uwsgi_cron_delta->tm_mon % abs(ucron->month)) == 0) {
-					uc_month = uwsgi_cron_delta->tm_mon;
-				}
-			}
-			if (ucron->day < 0) {
-				if ((uwsgi_cron_delta->tm_mday % abs(ucron->day)) == 0) {
-					uc_day = uwsgi_cron_delta->tm_mday;
-				}
-			}
-			if (ucron->week < 0) {
-				if ((uwsgi_cron_delta->tm_wday % abs(ucron->week)) == 0) {
-					uc_week = uwsgi_cron_delta->tm_wday;
-				}
-			}
-
-			int run_task = 0;
-			// mday and wday are ORed
-			if (ucron->day >= 0 && ucron->week >= 0) {
-				if (uwsgi_cron_delta->tm_min == uc_minute && uwsgi_cron_delta->tm_hour == uc_hour && uwsgi_cron_delta->tm_mon == uc_month && (uwsgi_cron_delta->tm_mday == uc_day || uwsgi_cron_delta->tm_wday == uc_week)) {
-					run_task = 1;
-				}
-			}
-			else {
-				if (uwsgi_cron_delta->tm_min == uc_minute && uwsgi_cron_delta->tm_hour == uc_hour && uwsgi_cron_delta->tm_mon == uc_month && uwsgi_cron_delta->tm_mday == uc_day && uwsgi_cron_delta->tm_wday == uc_week) {
-					run_task = 1;
-				}
-			}
-
-
-			if (run_task == 1) {
-				// date match, signal it ?
-				if (now - ucron->last_job > 60) {
-					uwsgi_route_signal(ucron->sig);
-					ucron->last_job = now;
-				}
-			}
-
-		}
-		uwsgi_unlock(uwsgi.cron_table_lock);
-	}
-	else {
-		uwsgi_error("localtime()");
-	}
-
-}
-
-void uwsgi_manage_command_cron(time_t now) {
-
-	struct tm *uwsgi_cron_delta;
-
-	struct uwsgi_cron *current_cron = uwsgi.crons;
-	int uc_minute, uc_hour, uc_day, uc_month, uc_week;
-
-	uwsgi_cron_delta = localtime(&now);
-
-
-	if (!uwsgi_cron_delta) {
-		uwsgi_error("localtime()");
-		return;
-	}
-
-	// fix month
-	uwsgi_cron_delta->tm_mon++;
-
-	while (current_cron) {
-
-		uc_minute = current_cron->minute;
-		uc_hour = current_cron->hour;
-		uc_day = current_cron->day;
-		uc_month = current_cron->month;
-		uc_week = current_cron->week;
-
-		// negative values as interval -1 = * , -5 = */5
-		if (current_cron->minute < 0) {
-			if ((uwsgi_cron_delta->tm_min % abs(current_cron->minute)) == 0) {
-				uc_minute = uwsgi_cron_delta->tm_min;
-			}
-		}
-		if (current_cron->hour < 0) {
-			if ((uwsgi_cron_delta->tm_hour % abs(current_cron->hour)) == 0) {
-				uc_hour = uwsgi_cron_delta->tm_hour;
-			}
-		}
-		if (current_cron->month < 0) {
-			if ((uwsgi_cron_delta->tm_mon % abs(current_cron->month)) == 0) {
-				uc_month = uwsgi_cron_delta->tm_mon;
-			}
-		}
-		if (current_cron->day < 0) {
-			if ((uwsgi_cron_delta->tm_mday % abs(current_cron->day)) == 0) {
-				uc_day = uwsgi_cron_delta->tm_mday;
-			}
-		}
-		if (current_cron->week < 0) {
-			if ((uwsgi_cron_delta->tm_wday % abs(current_cron->week)) == 0) {
-				uc_week = uwsgi_cron_delta->tm_wday;
-			}
-		}
-
-		int run_task = 0;
-		// mday and wday are ORed
-		if (current_cron->day >= 0 && current_cron->week >= 0) {
-			if (uwsgi_cron_delta->tm_min == uc_minute && uwsgi_cron_delta->tm_hour == uc_hour && uwsgi_cron_delta->tm_mon == uc_month && (uwsgi_cron_delta->tm_mday == uc_day || uwsgi_cron_delta->tm_wday == uc_week)) {
-				run_task = 1;
-			}
-		}
-		else {
-			if (uwsgi_cron_delta->tm_min == uc_minute && uwsgi_cron_delta->tm_hour == uc_hour && uwsgi_cron_delta->tm_mon == uc_month && uwsgi_cron_delta->tm_mday == uc_day && uwsgi_cron_delta->tm_wday == uc_week) {
-				run_task = 1;
-			}
-		}
-
-
-		if (run_task == 1) {
-
-			// date match, run command ?
-			if (now - current_cron->last_job > 60) {
-				//call command
-				if (current_cron->command) {
-					if (uwsgi_run_command(current_cron->command, NULL, -1) >= 0) {
-						uwsgi_log_verbose("[uWSGI-cron] running %s\n", current_cron->command);
-					}
-				}
-				current_cron->last_job = now;
-			}
-		}
-
-
-
-		current_cron = current_cron->next;
-	}
-
-
-}
-
 struct uwsgi_stats *uwsgi_master_generate_stats() {
 
 	int i;
@@ -940,7 +778,10 @@ struct uwsgi_stats *uwsgi_master_generate_stats() {
 			if (uwsgi_stats_keylong_comma(us, "miss", (unsigned long long) uc->miss))
 				goto end;
 
-			if (uwsgi_stats_keylong(us, "full", (unsigned long long) uc->full))
+			if (uwsgi_stats_keylong_comma(us, "full", (unsigned long long) uc->full))
+				goto end;
+
+			if (uwsgi_stats_keylong(us, "last_modified_at", (unsigned long long) uc->last_modified_at))
 				goto end;
 
 			if (uwsgi_stats_object_close(us))
@@ -1485,3 +1326,58 @@ void uwsgi_master_fix_request_counters() {
 	uwsgi.workers[0].requests = total_counter;
 }
 
+
+int uwsgi_cron_task_needs_execution(struct tm *uwsgi_cron_delta, int minute, int hour, int day, int month, int week) {
+
+	int uc_minute, uc_hour, uc_day, uc_month, uc_week;
+
+	uc_minute = minute;
+	uc_hour = hour;
+	uc_day = day;
+	uc_month = month;
+	uc_week = week;
+
+	// negative values as interval -1 = * , -5 = */5
+	if (minute < 0) {
+		if ((uwsgi_cron_delta->tm_min % abs(minute)) == 0) {
+			uc_minute = uwsgi_cron_delta->tm_min;
+		}
+	}
+	if (hour < 0) {
+		if ((uwsgi_cron_delta->tm_hour % abs(hour)) == 0) {
+			uc_hour = uwsgi_cron_delta->tm_hour;
+		}
+	}
+	if (month < 0) {
+		if ((uwsgi_cron_delta->tm_mon % abs(month)) == 0) {
+			uc_month = uwsgi_cron_delta->tm_mon;
+		}
+	}
+	if (day < 0) {
+		if ((uwsgi_cron_delta->tm_mday % abs(day)) == 0) {
+			uc_day = uwsgi_cron_delta->tm_mday;
+		}
+	}
+	if (week < 0) {
+		if ((uwsgi_cron_delta->tm_wday % abs(week)) == 0) {
+			uc_week = uwsgi_cron_delta->tm_wday;
+		}
+	}
+
+	int run_task = 0;
+	// mday and wday are ORed
+	if (day >= 0 && week >= 0) {
+		if (uwsgi_cron_delta->tm_min == uc_minute && uwsgi_cron_delta->tm_hour == uc_hour && uwsgi_cron_delta->tm_mon == uc_month && (uwsgi_cron_delta->tm_mday == uc_day || uwsgi_cron_delta->tm_wday == uc_week)) {
+			run_task = 1;
+		}
+	}
+	else {
+		if (uwsgi_cron_delta->tm_min == uc_minute && uwsgi_cron_delta->tm_hour == uc_hour && uwsgi_cron_delta->tm_mon == uc_month && uwsgi_cron_delta->tm_mday == uc_day && uwsgi_cron_delta->tm_wday == uc_week) {
+			run_task = 1;
+		}
+	}
+
+	return run_task;
+
+}
+
