@@ -1067,6 +1067,29 @@ static int uwsgi_route_condition_startswith(struct wsgi_request *wsgi_req, struc
         return 0;
 }
 
+static int uwsgi_route_condition_contains(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
+        char *semicolon = memchr(ur->subject_str, ';', ur->subject_str_len);
+        if (!semicolon) return 0;
+
+        struct uwsgi_buffer *ub = uwsgi_routing_translate(wsgi_req, ur, NULL, 0, ur->subject_str, semicolon - ur->subject_str);
+        if (!ub) return -1;
+
+        struct uwsgi_buffer *ub2 = uwsgi_routing_translate(wsgi_req, ur, NULL, 0, semicolon+1, ur->subject_str_len - ((semicolon+1) - ur->subject_str));
+        if (!ub2) {
+                uwsgi_buffer_destroy(ub);
+                return -1;
+        }
+
+        if(uwsgi_contains_n(ub->buf, ub->pos, ub2->buf, ub2->pos)) {
+                uwsgi_buffer_destroy(ub);
+                uwsgi_buffer_destroy(ub2);
+                return 1;
+        }
+        uwsgi_buffer_destroy(ub);
+        uwsgi_buffer_destroy(ub2);
+        return 0;
+}
+
 static int uwsgi_route_condition_endswith(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
         char *semicolon = memchr(ur->subject_str, ';', ur->subject_str_len);
         if (!semicolon) return 0;
@@ -1143,6 +1166,14 @@ static char *uwsgi_route_var_uwsgi(struct wsgi_request *wsgi_req, char *key, uin
 	return ret;
 }
 
+static char *uwsgi_route_var_mime(struct wsgi_request *wsgi_req, char *key, uint16_t keylen, uint16_t *vallen) {
+	size_t mime_type_len = 0;
+        char *ret = uwsgi_get_mime_type(key, keylen, &mime_type_len);
+	if (ret) *vallen = mime_type_len;
+        return ret;
+}
+
+
 static char *uwsgi_route_var_time(struct wsgi_request *wsgi_req, char *key, uint16_t keylen, uint16_t *vallen) {
         char *ret = NULL;
         if (!uwsgi_strncmp(key, keylen, "unix", 4)) {
@@ -1152,6 +1183,29 @@ static char *uwsgi_route_var_time(struct wsgi_request *wsgi_req, char *key, uint
         return ret;
 }
 
+static char *uwsgi_route_var_base64(struct wsgi_request *wsgi_req, char *key, uint16_t keylen, uint16_t *vallen) {
+	char *ret = NULL;
+	uint16_t var_vallen = 0;
+        char *var_value = uwsgi_get_var(wsgi_req, key, keylen, &var_vallen);
+	if (var_value) {
+		size_t b64_len = 0;
+		ret = uwsgi_base64_encode(var_value, var_vallen, &b64_len);
+		*vallen = b64_len;
+	}
+        return ret;
+}
+
+static char *uwsgi_route_var_hex(struct wsgi_request *wsgi_req, char *key, uint16_t keylen, uint16_t *vallen) {
+        char *ret = NULL;
+        uint16_t var_vallen = 0;
+        char *var_value = uwsgi_get_var(wsgi_req, key, keylen, &var_vallen);
+        if (var_value) {
+                ret = uwsgi_str_to_hex(var_value, var_vallen);
+                *vallen = var_vallen*2;
+        }
+        return ret;
+}
+
 #ifdef UWSGI_MATHEVAL
 static char *uwsgi_route_var_math(struct wsgi_request *wsgi_req, char *key, uint16_t keylen, uint16_t *vallen) {
 	char *ret = NULL;
@@ -1238,6 +1292,8 @@ void uwsgi_register_embedded_routers() {
         uwsgi_register_route_condition(">=", uwsgi_route_condition_higherequal);
         uwsgi_register_route_condition("islowerequal", uwsgi_route_condition_lowerequal);
         uwsgi_register_route_condition("<=", uwsgi_route_condition_lowerequal);
+        uwsgi_register_route_condition("contains", uwsgi_route_condition_contains);
+        uwsgi_register_route_condition("contain", uwsgi_route_condition_contains);
 #ifdef UWSGI_SSL
         uwsgi_register_route_condition("lord", uwsgi_route_condition_lord);
 #endif
@@ -1246,6 +1302,7 @@ void uwsgi_register_embedded_routers() {
 
         uwsgi_register_route_var("cookie", uwsgi_get_cookie);
         uwsgi_register_route_var("qs", uwsgi_get_qs);
+        uwsgi_register_route_var("mime", uwsgi_route_var_mime);
         struct uwsgi_route_var *urv = uwsgi_register_route_var("uwsgi", uwsgi_route_var_uwsgi);
 	urv->need_free = 1;
         urv = uwsgi_register_route_var("time", uwsgi_route_var_time);
@@ -1254,6 +1311,11 @@ void uwsgi_register_embedded_routers() {
         urv = uwsgi_register_route_var("math", uwsgi_route_var_math);
 	urv->need_free = 1;
 #endif
+        urv = uwsgi_register_route_var("base64", uwsgi_route_var_base64);
+	urv->need_free = 1;
+
+        urv = uwsgi_register_route_var("hex", uwsgi_route_var_hex);
+	urv->need_free = 1;
 }
 
 struct uwsgi_router *uwsgi_register_router(char *name, int (*func) (struct uwsgi_route *, char *)) {
