@@ -222,12 +222,11 @@ void uwsgi_set_cgroup() {
 			uwsgi_error_open(cgroup_taskfile);
 			exit(1);
 		}
-		if (fprintf(cgroup, "%d", (int) getpid()) <= 0) {
-			uwsgi_log("could not set cgroup\n");
+		if (fprintf(cgroup, "%d\n", (int) getpid()) <= 0 || ferror(cgroup) || fclose(cgroup)) {
+			uwsgi_error("could not set cgroup");
 			exit(1);
 		}
 		uwsgi_log("assigned process %d to cgroup %s\n", (int) getpid(), cgroup_taskfile);
-		fclose(cgroup);
 		free(cgroup_taskfile);
 
 
@@ -248,11 +247,10 @@ void uwsgi_set_cgroup() {
 			cgroup_taskfile = uwsgi_concat3(usl->value, "/", uslo->value);
 			cgroup = fopen(cgroup_taskfile, "w");
 			if (cgroup) {
-				if (fprintf(cgroup, "%s\n", cgroup_opt) < 0) {
+				if (fprintf(cgroup, "%s\n", cgroup_opt) <= 0 || ferror(cgroup) || fclose(cgroup)) {
 					uwsgi_log("could not set cgroup option %s to %s\n", uslo->value, cgroup_opt);
 					exit(1);
 				}
-				fclose(cgroup);
 				uwsgi_log("set %s to %s\n", cgroup_opt, cgroup_taskfile);
 			}
 			free(cgroup_taskfile);
@@ -576,6 +574,23 @@ void uwsgi_close_request(struct wsgi_request *wsgi_req) {
 	int tmp_id;
 	uint64_t tmp_rt, rss = 0, vsz = 0;
 
+	// flush the response buf (if any)
+	if (wsgi_req->response_buffer) {
+		// send the body only if transformations are successfull
+		struct uwsgi_buffer *ub = wsgi_req->response_buffer;
+		if (uwsgi_apply_transformations(wsgi_req) == 0) {
+			// the response_buffer could be changed by translations
+			ub = wsgi_req->response_buffer;
+			// force true write
+			wsgi_req->response_buffer = NULL;
+			uwsgi_response_write_body_do(wsgi_req, ub->buf, ub->pos);
+		}
+		else {
+			wsgi_req->write_errors++;
+		}
+		uwsgi_buffer_destroy(ub);
+	}
+
 	// check if headers should be sent
 	if (wsgi_req->headers) {
 		if (!wsgi_req->headers_sent && !wsgi_req->headers_size && !wsgi_req->response_size) {
@@ -599,7 +614,6 @@ void uwsgi_close_request(struct wsgi_request *wsgi_req) {
 		uwsgi.workers[uwsgi.mywid].rss_size = rss;
 	}
 
-
 	// close the connection with the client
 	if (!wsgi_req->fd_closed) {
 		// NOTE, if we close the socket before receiving eventually sent data, socket layer will send a RST
@@ -622,25 +636,6 @@ void uwsgi_close_request(struct wsgi_request *wsgi_req) {
 		free(wsgi_req->proto_parser_buf);
 	}
 
-	// do we need to store the response in the cache
-	if (wsgi_req->cache_it) {
-		// only response with a body and a status code of 200 can be cached (TODO allows it to be configurable)
-		if (wsgi_req->cached_response && wsgi_req->status == 200 && wsgi_req->response_size > 0) {
-			uwsgi_cache_magic_set(wsgi_req->cache_it->buf, wsgi_req->cache_it->pos,
-				wsgi_req->cached_response->buf, wsgi_req->cached_response->pos, wsgi_req->cache_it_expires, UWSGI_CACHE_FLAG_UPDATE, wsgi_req->cache_it_to ? wsgi_req->cache_it_to->buf : NULL);
-		}
-		uwsgi_buffer_destroy(wsgi_req->cache_it);
-			
-	}
-
-	if (wsgi_req->cache_it_to) {
-		uwsgi_buffer_destroy(wsgi_req->cache_it_to);
-	}
-
-	if (wsgi_req->cached_response) {
-		uwsgi_buffer_destroy(wsgi_req->cached_response);
-	}
-
 	if (!wsgi_req->do_not_account) {
 		uwsgi.workers[0].requests++;
 		uwsgi.workers[uwsgi.mywid].requests++;
@@ -1141,11 +1136,36 @@ void parse_sys_envs(char **envs) {
 }
 
 // get the application id
-int uwsgi_get_app_id(char *app_name, int app_name_len, int modifier1) {
+int uwsgi_get_app_id(struct wsgi_request *wsgi_req, char *key, uint16_t key_len, int modifier1) {
 
 	int i;
 	struct stat st;
 	int found;
+	int free_appname = 0;
+
+	char *app_name = key;
+	uint16_t app_name_len = key_len;
+
+	if (!app_name && wsgi_req) {
+		app_name = wsgi_req->appid;
+		app_name_len = wsgi_req->appid_len;
+		if (app_name_len == 0) {
+			if (!uwsgi.ignore_script_name) {
+				app_name = wsgi_req->script_name;
+				app_name_len = wsgi_req->script_name_len;
+			}
+
+			if (uwsgi.vhost) {
+                        	app_name = uwsgi_concat3n(wsgi_req->host, wsgi_req->host_len, "|",1, wsgi_req->script_name, wsgi_req->script_name_len);
+                        	app_name_len = wsgi_req->host_len + 1 + wsgi_req->script_name_len;
+#ifdef UWSGI_DEBUG
+                        	uwsgi_debug("VirtualHost KEY=%.*s\n", wsgi_req->appid_len, wsgi_req->appid);
+#endif
+                        	free_appname = 1;
+			}
+                }
+        }
+
 
 	for (i = 0; i < uwsgi_apps_cnt; i++) {
 		// reset check
@@ -1177,14 +1197,19 @@ int uwsgi_get_app_id(char *app_name, int app_name_len, int modifier1) {
 					}
 				}
 			}
-			if (modifier1 == -1)
+			if (modifier1 == -1) {
+				if (free_appname) free(app_name);
 				return i;
-			if (modifier1 == uwsgi_apps[i].modifier1)
+			}
+			if (modifier1 == uwsgi_apps[i].modifier1) {
+				if (free_appname) free(app_name);
 				return i;
+			}
 		}
 	}
 
 	if (!uwsgi.no_default_app) {
+		if (free_appname) free(app_name);
 		return uwsgi.default_app;
 	}
 
@@ -3236,10 +3261,9 @@ void uwsgi_write_pidfile(char *pidfile_name) {
 		uwsgi_error_open(pidfile_name);
 		exit(1);
 	}
-	if (fprintf(pidfile, "%d\n", (int) getpid()) < 0) {
+	if (fprintf(pidfile, "%d\n", (int) getpid()) <= 0 || ferror(pidfile) || fclose(pidfile)) {
 		uwsgi_log("could not write pidfile.\n");
 	}
-	fclose(pidfile);
 }
 
 char *uwsgi_expand_path(char *dir, int dir_len, char *ptr) {
@@ -3851,3 +3875,33 @@ char *uwsgi_get_header(struct wsgi_request *wsgi_req, char *hh, uint16_t len, ui
 	return value;
 	
 }
+
+static char *uwsgi_hex_table[] = {
+	"00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "0A", "0B", "0C", "0D", "0E", "0F",
+	"10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "1A", "1B", "1C", "1D", "1E", "1F",
+	"20", "21", "22", "23", "24", "25", "26", "27", "28", "29", "2A", "2B", "2C", "2D", "2E", "2F",
+	"30", "31", "32", "33", "34", "35", "36", "37", "38", "39", "3A", "3B", "3C", "3D", "3E", "3F",
+	"40", "41", "42", "43", "44", "45", "46", "47", "48", "49", "4A", "4B", "4C", "4D", "4E", "4F",
+	"50", "51", "52", "53", "54", "55", "56", "57", "58", "59", "5A", "5B", "5C", "5D", "5E", "5F",
+	"60", "61", "62", "63", "64", "65", "66", "67", "68", "69", "6A", "6B", "6C", "6D", "6E", "6F",
+	"70", "71", "72", "73", "74", "75", "76", "77", "78", "79", "7A", "7B", "7C", "7D", "7E", "7F",
+	"80", "81", "82", "83", "84", "85", "86", "87", "88", "89", "8A", "8B", "8C", "8D", "8E", "8F",
+	"90", "91", "92", "93", "94", "95", "96", "97", "98", "99", "9A", "9B", "9C", "9D", "9E", "9F",
+	"A0", "A1", "A2", "A3", "A4", "A5", "A6", "A7", "A8", "A9", "AA", "AB", "AC", "AD", "AE", "AF",
+	"B0", "B1", "B2", "B3", "B4", "B5", "B6", "B7", "B8", "B9", "BA", "BB", "BC", "BD", "BE", "BF",
+	"C0", "C1", "C2", "C3", "C4", "C5", "C6", "C7", "C8", "C9", "CA", "CB", "CC", "CD", "CE", "CF",
+	"D0", "D1", "D2", "D3", "D4", "D5", "D6", "D7", "D8", "D9", "DA", "DB", "DC", "DD", "DE", "DF",
+	"E0", "E1", "E2", "E3", "E4", "E5", "E6", "E7", "E8", "E9", "EA", "EB", "EC", "ED", "EE", "EF",
+	"F0", "F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "FA", "FB", "FC", "FD", "FE", "FF",
+};
+
+char *uwsgi_str_to_hex(char *src, size_t slen) {
+	char *dst = uwsgi_malloc(slen*2);
+	char *ptr = dst;
+	size_t i;
+	for(i=0;i<slen;i++) {
+		memcpy(ptr, uwsgi_hex_table[ (int) src[i] ], 2);
+		ptr+=2;
+	}
+	return dst;
+}
