@@ -12,6 +12,17 @@ int uwsgi_response_add_content_length(struct wsgi_request *wsgi_req, uint64_t cl
 	return uwsgi_response_add_header(wsgi_req, "Content-Length", 14, buf, ret); 
 }
 
+int uwsgi_response_add_expires(struct wsgi_request *wsgi_req, uint64_t t) {
+	// 30+1
+        char expires[31];
+	int len = uwsgi_http_date((time_t) t, expires);
+	if (!len) {
+		wsgi_req->write_errors++;
+                return -1;
+        }
+	return uwsgi_response_add_header(wsgi_req, "Expires", 7, expires, len); 
+}
+
 int uwsgi_response_add_content_range(struct wsgi_request *wsgi_req, uint64_t start, uint64_t end, uint64_t cl) {
         char buf[6+(sizeof(UMAX64_STR)*3)+4];
 	if (end == 0) {
@@ -25,6 +36,12 @@ int uwsgi_response_add_content_range(struct wsgi_request *wsgi_req, uint64_t sta
         return uwsgi_response_add_header(wsgi_req, "Content-Range", 13, buf, ret);
 }
 
+int uwsgi_response_prepare_headers_int(struct wsgi_request *wsgi_req, int status) {
+	char status_str[11];
+	uwsgi_num2str2(status, status_str);
+	return uwsgi_response_prepare_headers(wsgi_req, status_str, 3);
+}
+
 // status could be NNN or NNN message
 int uwsgi_response_prepare_headers(struct wsgi_request *wsgi_req, char *status, uint16_t status_len) {
 
@@ -159,7 +176,8 @@ int uwsgi_response_write_body_do(struct wsgi_request *wsgi_req, char *buf, size_
 
 	if (wsgi_req->write_errors) return -1;
 
-	if (!wsgi_req->headers_sent) {
+	// do not commit headers until a response_buffer is available
+	if (!wsgi_req->response_buffer && !wsgi_req->headers_sent) {
 		int ret = uwsgi_response_write_headers_do(wsgi_req);
                 if (ret == UWSGI_OK) goto sendbody;
                 if (ret == UWSGI_AGAIN) return UWSGI_AGAIN;
@@ -170,6 +188,14 @@ int uwsgi_response_write_body_do(struct wsgi_request *wsgi_req, char *buf, size_
 sendbody:
 
 	if (len == 0) return UWSGI_OK;
+	
+	if (wsgi_req->response_buffer) {
+		if (uwsgi_buffer_append(wsgi_req->response_buffer, buf, len)) {
+			wsgi_req->write_errors++;
+			return -1;
+		}
+		return UWSGI_OK;
+	}
 
 	for(;;) {
 		int ret = wsgi_req->socket->proto_write(wsgi_req, buf, len);
@@ -196,18 +222,6 @@ sendbody:
 	// reset for the next write
         wsgi_req->write_pos = 0;
 
-	// now we need to check if the chunk must be stored
-	if (wsgi_req->cache_it) {
-		if (!wsgi_req->cached_response) {
-			wsgi_req->cached_response = uwsgi_buffer_new(len);
-		}
-		// if we are unable to append the buffer, we just stop caching it
-		if (uwsgi_buffer_append(wsgi_req->cached_response, buf, len)) {
-			uwsgi_buffer_destroy(wsgi_req->cache_it);
-			wsgi_req->cache_it = NULL;
-		}
-	}
-
 	return UWSGI_OK;	
 }
 
@@ -217,7 +231,10 @@ int uwsgi_response_sendfile_do(struct wsgi_request *wsgi_req, int fd, size_t pos
 
 	if (fd == wsgi_req->sendfile_fd) can_close = 0;
 
-	if (wsgi_req->write_errors) return -1;
+	if (wsgi_req->write_errors) {
+		if (can_close) close(fd);
+		return -1;
+	}
 
 	if (!wsgi_req->headers_sent) {
 		int ret = uwsgi_response_write_headers_do(wsgi_req);
@@ -238,7 +255,10 @@ sendfile:
 			if (can_close) close(fd);
 			return -1;
 		}
-		if (pos >= (size_t)st.st_size) return UWSGI_OK;
+		if (pos >= (size_t)st.st_size) {
+			if (can_close) close(fd);
+			return UWSGI_OK;
+		}
 		len = st.st_size;
 	}
 
@@ -290,6 +310,7 @@ sendfile:
 		if (ret == 0) {
                         uwsgi_log("uwsgi_response_sendfile_do() TIMEOUT !!!\n");
                         wsgi_req->write_errors++;
+			if (can_close) close(fd);
                         return -1;
                 }	
         }
