@@ -267,7 +267,7 @@ static MonoArray *uwsgi_mono_method_api_CacheGet(MonoString *key, MonoString *ca
 		c_cache = mono_string_to_utf8(cache);
 	}
 	uint64_t vallen = 0 ;
-	char *value = uwsgi_cache_magic_get(c_key, c_keylen, &vallen, c_cache);
+	char *value = uwsgi_cache_magic_get(c_key, c_keylen, &vallen, NULL, c_cache);
         if (value) {
 		MonoArray *ret = mono_array_new(mono_domain_get(), umono.byte_class, vallen);
 		char *buf = mono_array_addr(ret, char, 0);
@@ -540,7 +540,7 @@ static int uwsgi_mono_request(struct wsgi_request *wsgi_req) {
 		key_len = 0;
 	}
 
-        wsgi_req->app_id = uwsgi_get_app_id(key, key_len, mono_plugin.modifier1);
+        wsgi_req->app_id = uwsgi_get_app_id(NULL, key, key_len, mono_plugin.modifier1);
         // if it is -1, try to load a dynamic app
         if (wsgi_req->app_id == -1) {
         	if (uwsgi.threads > 1) {
@@ -548,7 +548,7 @@ static int uwsgi_mono_request(struct wsgi_request *wsgi_req) {
                 }
 
 		// check if the mean time, something changed		
-		wsgi_req->app_id = uwsgi_get_app_id(key, key_len, mono_plugin.modifier1);
+		wsgi_req->app_id = uwsgi_get_app_id(NULL, key, key_len, mono_plugin.modifier1);
 
 		if (wsgi_req->app_id == -1) {
                 	wsgi_req->app_id = uwsgi_mono_create_app(key, key_len, key, key_len, 0);
