@@ -15,6 +15,10 @@
 
 	xslt:doc=<path1>,stylesheet=<path2>,params=<params>
 
+	As transformation
+
+	toxslt:stylesheet=<path2>,params=<params>
+
 */
 
 struct uwsgi_xslt_config {
@@ -37,6 +41,12 @@ struct uwsgi_router_xslt_conf {
 	uint16_t content_type_len;
 };
 
+struct uwsgi_transformation_xslt_conf {
+	struct uwsgi_buffer *stylesheet;
+        struct uwsgi_buffer *params;
+        struct uwsgi_buffer *content_type;
+};
+
 struct uwsgi_option uwsgi_xslt_options[] = {
 	{"xslt-docroot", required_argument, 0, "add a document_root for xslt processing", uwsgi_opt_add_string_list, &uxslt.docroot, 0},
 	{"xslt-ext", required_argument, 0, "search for xslt stylesheets with the specified extension", uwsgi_opt_add_string_list, &uxslt.ext, 0},
@@ -46,7 +56,7 @@ struct uwsgi_option uwsgi_xslt_options[] = {
 	{NULL, 0, 0, NULL, NULL, NULL, 0},
 };
 
-static char *uwsgi_xslt_apply(char *xmlfile, char *xsltfile, char *params, int *rlen) {
+static char *uwsgi_xslt_apply(xmlDoc *doc, char *xsltfile, char *params, int *rlen) {
 
 	char **vparams = NULL;
 	char *tmp_params = NULL;
@@ -79,19 +89,8 @@ static char *uwsgi_xslt_apply(char *xmlfile, char *xsltfile, char *params, int *
 	xmlSubstituteEntitiesDefault(1);
 	xmlLoadExtDtdDefaultValue = 1;
 
-        xmlDocPtr doc = xmlParseFile(xmlfile);
-        if (!doc) {
-		if (vparams) {
-			int i; for(i=1;i<(count*2);i+=2) free(vparams[i]);
-			free(tmp_params);
-			free(vparams);
-		}
-		return NULL;
-	}
-
         xsltStylesheetPtr ss = xsltParseStylesheetFile((const xmlChar *) xsltfile);
         if (!ss) {
-		xmlFreeDoc(doc);
 		if (vparams) {
 			int i; for(i=1;i<(count*2);i+=2) free(vparams[i]);
 			free(tmp_params);
@@ -103,7 +102,6 @@ static char *uwsgi_xslt_apply(char *xmlfile, char *xsltfile, char *params, int *
         xmlDocPtr res = xsltApplyStylesheet(ss, doc, (const char **) vparams);
 	if (!res) {
 		xsltFreeStylesheet(ss);
-		xmlFreeDoc(doc);
 		if (vparams) {
 			int i; for(i=1;i<(count*2);i+=2) free(vparams[i]);
 			free(tmp_params);
@@ -116,7 +114,6 @@ static char *uwsgi_xslt_apply(char *xmlfile, char *xsltfile, char *params, int *
         int ret = xsltSaveResultToString(&output, rlen, res, ss);
 	xsltFreeStylesheet(ss);
 	xmlFreeDoc(res);
-	xmlFreeDoc(doc);
 	if (vparams) {
 		int i; for(i=1;i<(count*2);i+=2) free(vparams[i]);
 		free(tmp_params);
@@ -279,7 +276,13 @@ apply:
 		params = uwsgi_concat2n(wsgi_req->query_string, wsgi_req->query_string_len, "", 0);
 	}
 	// we have both the file and the stylesheet, let's run the engine
-	output = uwsgi_xslt_apply(filename, stylesheet, params, &output_rlen);
+	xmlDoc *doc = xmlParseFile(xmlfile);
+	if (!doc) {
+		uwsgi_500(wsgi_req);
+                return UWSGI_OK;
+	} 
+	output = uwsgi_xslt_apply(doc, stylesheet, params, &output_rlen);
+	xmlFree(doc);
 	if (params) free(params);
 	if (!output) {
 		uwsgi_500(wsgi_req);
@@ -313,6 +316,72 @@ static void uwsgi_xslt_log(struct wsgi_request *wsgi_req) {
 	log_request(wsgi_req);
 }
 
+static int transform_toxslt(struct wsgi_request *wsgi_req, struct uwsgi_buffer *ub, struct uwsgi_buffer **new, void *data) {
+	int ret = -1;
+        struct uwsgi_transformation_xslt_conf *utxc = (struct uwsgi_transformation_xslt_conf *) data;
+
+	xmlDoc *doc = xmlReadMemory(ub->buf, ub->pos, NULL, NULL, 0);
+	if (!doc) goto end;
+
+	int rlen;
+        char *output = uwsgi_xslt_apply( doc, utxc->stylesheet->buf, utxc->params ? utxc->params->buf : NULL, &rlen);
+	if (!output) goto end;
+
+        if (uwsgi_response_prepare_headers_int(wsgi_req, wsgi_req->status)) goto end;
+        if (uwsgi_response_add_content_length(wsgi_req, rlen)) goto end;
+        if (uwsgi_response_add_content_type(wsgi_req, utxc->content_type->buf, utxc->content_type->pos)) goto end;
+
+	*new = uwsgi_buffer_new(rlen);
+	if (uwsgi_buffer_append(*new, output, rlen)) {
+		xmlFree(output);
+		uwsgi_buffer_destroy(*new);
+		*new = NULL;
+		goto end;
+	}
+	xmlFree(output);
+	ret = 0;
+
+end:
+	if (doc) xmlFreeDoc(doc);
+        if (utxc->stylesheet) uwsgi_buffer_destroy(utxc->stylesheet);
+        if (utxc->params) uwsgi_buffer_destroy(utxc->params);
+        if (utxc->content_type) uwsgi_buffer_destroy(utxc->content_type);
+	free(utxc);
+	return ret;
+}
+
+static int uwsgi_routing_func_toxslt(struct wsgi_request *wsgi_req, struct uwsgi_route *ur){
+
+        struct uwsgi_router_xslt_conf *urxc = (struct uwsgi_router_xslt_conf *) ur->data2;
+	struct uwsgi_transformation_xslt_conf *utxc = uwsgi_calloc(sizeof(struct uwsgi_transformation_xslt_conf));
+
+        char **subject = (char **) (((char *)(wsgi_req))+ur->subject);
+        uint16_t *subject_len = (uint16_t *) (((char *)(wsgi_req))+ur->subject_len);
+
+        utxc->stylesheet = uwsgi_routing_translate(wsgi_req, ur, *subject, *subject_len, urxc->stylesheet, urxc->stylesheet_len);
+        if (!utxc->stylesheet) goto end;
+
+        if (urxc->params) {
+                utxc->params = uwsgi_routing_translate(wsgi_req, ur, *subject, *subject_len, urxc->params, urxc->params_len);
+                if (!utxc->params) goto end;
+        }
+
+        if (urxc->content_type) {
+                utxc->content_type = uwsgi_routing_translate(wsgi_req, ur, *subject, *subject_len, urxc->content_type, urxc->content_type_len);
+                if (!utxc->content_type) goto end;
+        }
+
+	uwsgi_add_transformation(wsgi_req, transform_toxslt, utxc);
+	return UWSGI_ROUTE_NEXT;
+end:
+        if (utxc->stylesheet) uwsgi_buffer_destroy(utxc->stylesheet);
+        if (utxc->params) uwsgi_buffer_destroy(utxc->params);
+        if (utxc->content_type) uwsgi_buffer_destroy(utxc->content_type);
+	free(utxc);
+        return UWSGI_ROUTE_BREAK;
+}
+
+
 static int uwsgi_routing_func_xslt(struct wsgi_request *wsgi_req, struct uwsgi_route *ur){
 
         struct uwsgi_router_xslt_conf *urxc = (struct uwsgi_router_xslt_conf *) ur->data2;
@@ -341,7 +410,10 @@ static int uwsgi_routing_func_xslt(struct wsgi_request *wsgi_req, struct uwsgi_r
 	}
 
 	int rlen;
-        char *output = uwsgi_xslt_apply( ub_doc->buf, ub_stylesheet->buf, ub_params ? ub_params->buf : NULL, &rlen);
+	xmlDoc *doc = xmlParseFile(ub_doc->buf) ;
+	if (!doc) goto end;
+        char *output = uwsgi_xslt_apply(doc, ub_stylesheet->buf, ub_params ? ub_params->buf : NULL, &rlen);
+	xmlFree(doc);
 	if (!output) goto end;
 
         if (uwsgi_response_prepare_headers(wsgi_req, "200 OK", 6)) goto end;
@@ -390,9 +462,40 @@ static int uwsgi_router_xslt(struct uwsgi_route *ur, char *args) {
         return 0;
 }
 
+static int uwsgi_router_toxslt(struct uwsgi_route *ur, char *args) {
+        ur->func = uwsgi_routing_func_toxslt;
+        ur->data = args;
+        ur->data_len = strlen(args);
+        struct uwsgi_router_xslt_conf *urxc = uwsgi_calloc(sizeof(struct uwsgi_router_xslt_conf));
+        if (uwsgi_kvlist_parse(ur->data, ur->data_len, ',', '=',
+                        "stylesheet", &urxc->stylesheet,
+                        "content_type", &urxc->content_type,
+                        "params", &urxc->params,
+                        NULL)) {
+                        uwsgi_log("invalid route syntax: %s\n", args);
+                        exit(1);
+        }
+
+        if (!urxc->stylesheet) {
+                uwsgi_log("invalid route/transformation syntax: you need to specify a stylesheet\n");
+                exit(1);
+        }
+
+        urxc->stylesheet_len = strlen(urxc->stylesheet);
+
+        if (urxc->params) urxc->params_len = strlen(urxc->params);
+        if (!urxc->content_type) urxc->content_type = "text/html";
+        urxc->content_type_len = strlen(urxc->content_type);
+        ur->data2 = urxc;
+        return 0;
+}
+
+
+
 
 static void router_xslt_register() {
         uwsgi_register_router("xslt", uwsgi_router_xslt);
+        uwsgi_register_router("toxslt", uwsgi_router_toxslt);
 }
 
 
