@@ -713,6 +713,9 @@ struct uwsgi_cache {
 	uint64_t n_items;
 	struct uwsgi_cache_item *items;
 
+	uint8_t use_last_modified;
+	time_t last_modified_at;
+
 	void *data;
 
 	uint8_t no_expire;
@@ -1143,6 +1146,18 @@ struct uwsgi_alarm {
 	struct uwsgi_alarm *next;
 };
 
+struct uwsgi_alarm_fd {
+	int fd;
+	size_t buf_len;
+	void *buf;
+	char *msg;
+	size_t msg_len;
+	struct uwsgi_alarm_instance *alarm;
+	struct uwsgi_alarm_fd *next;
+};
+
+struct uwsgi_alarm_fd *uwsgi_add_alarm_fd(int, char *, size_t, char *, size_t);
+
 #ifdef UWSGI_PCRE
 struct uwsgi_alarm_ll {
 	struct uwsgi_alarm_instance *alarm;
@@ -1179,6 +1194,12 @@ struct uwsgi_logvar {
 	struct uwsgi_logvar *next;
 };
 
+struct uwsgi_transformation {
+	int (*func)(struct wsgi_request *, struct uwsgi_buffer *, struct uwsgi_buffer **, void *);
+	void *data;
+	struct uwsgi_transformation *next;
+};
+
 struct wsgi_request {
 	int fd;
 	struct uwsgi_header *uh;
@@ -1392,10 +1413,8 @@ struct wsgi_request {
 	// internal routing goto instruction
 	uint32_t route_goto;
 
-	struct uwsgi_buffer *cache_it;
-	struct uwsgi_buffer *cache_it_to;
-	uint64_t cache_it_expires;
-	struct uwsgi_buffer *cached_response;
+	struct uwsgi_buffer *response_buffer;
+	struct uwsgi_transformation *transformations;
 
 	struct msghdr msg;
 	union {
@@ -1823,6 +1842,8 @@ struct uwsgi_server {
 	uint64_t alarm_msg_size;
 	struct uwsgi_string_list *alarm_list;
 	struct uwsgi_string_list *alarm_logs_list;
+	struct uwsgi_alarm_fd *alarm_fds;
+	struct uwsgi_string_list *alarm_fd_list;
 	struct uwsgi_alarm *alarms;
 	struct uwsgi_alarm_instance *alarm_instances;
 	struct uwsgi_alarm_log *alarm_logs;
@@ -1831,6 +1852,9 @@ struct uwsgi_server {
 	int threaded_logger;
 	pthread_mutex_t threaded_logger_lock;
 
+	int *safe_fds;
+	int safe_fds_cnt;
+
 	int daemons_honour_stdin;
 	struct uwsgi_daemon *daemons;
 	int daemons_cnt;
@@ -1863,6 +1887,7 @@ struct uwsgi_server {
 	struct uwsgi_string_list *cache2;
 	int cache_setup;
 	int locking_setup;
+	int cache_use_last_modified;
 
 	struct uwsgi_dyn_dict *static_expires_type;
 	struct uwsgi_dyn_dict *static_expires_type_mtime;
@@ -2079,6 +2104,7 @@ struct uwsgi_server {
 
 	int no_orphans;
 	int skip_zero;
+	int skip_atexit;
 
 	char *chdir;
 	char *chdir2;
@@ -2104,6 +2130,7 @@ struct uwsgi_server {
 	char *ns_net;
 	struct uwsgi_string_list *ns_keep_mount;
 #endif
+	struct uwsgi_string_list *file_write_list;
 
 	char *protocol;
 
@@ -2172,6 +2199,9 @@ struct uwsgi_server {
 	size_t rwlock_size;
 
 	struct uwsgi_string_list *load_file_in_cache;
+#ifdef UWSGI_ZLIB
+	struct uwsgi_string_list *load_file_in_cache_gzip;
+#endif
 	char *use_check_cache;
 	struct uwsgi_cache *check_cache;
 	struct uwsgi_cache *caches;
@@ -2279,8 +2309,8 @@ struct uwsgi_rpc {
 	char name[UMAX8];
 	void *func;
 	uint8_t args;
-	uint8_t modifier1;
 	uint8_t shared;
+	struct uwsgi_plugin *plugin;
 };
 
 struct uwsgi_signal_entry {
@@ -2314,6 +2344,10 @@ struct uwsgi_cron {
 	uint8_t sig;
 
 	char *command;
+#ifdef UWSGI_SSL
+	char *legion;
+#endif
+	void (*func)(struct uwsgi_cron *, time_t);
 
 	struct uwsgi_cron *next;
 };
@@ -2646,7 +2680,7 @@ char *uwsgi_concat4(char *, char *, char *, char *);
 char *uwsgi_concat4n(char *, int, char *, int, char *, int, char *, int);
 
 
-int uwsgi_get_app_id(char *, int, int);
+int uwsgi_get_app_id(struct wsgi_request *, char *, uint16_t, int);
 char *uwsgi_strncopy(char *, int);
 
 int master_loop(char **, char **);
@@ -2698,6 +2732,7 @@ ssize_t uwsgi_send_message(int, uint8_t, uint8_t, char *, uint16_t, int, ssize_t
 int uwsgi_cache_set2(struct uwsgi_cache *, char *, uint16_t, char *, uint64_t, uint64_t, uint64_t);
 int uwsgi_cache_del2(struct uwsgi_cache *, char *, uint16_t, uint64_t, uint16_t);
 char *uwsgi_cache_get2(struct uwsgi_cache *, char *, uint16_t, uint64_t *);
+char *uwsgi_cache_get3(struct uwsgi_cache *, char *, uint16_t, uint64_t *, uint64_t *);
 uint32_t uwsgi_cache_exists2(struct uwsgi_cache *, char *, uint16_t);
 struct uwsgi_cache *uwsgi_cache_create(char *);
 struct uwsgi_cache *uwsgi_cache_by_name(char *);
@@ -2750,7 +2785,7 @@ void uwsgi_route_signal(uint8_t);
 
 int uwsgi_start(void *);
 
-int uwsgi_register_rpc(char *, uint8_t, uint8_t, void *);
+int uwsgi_register_rpc(char *, struct uwsgi_plugin *, uint8_t, void *);
 uint16_t uwsgi_rpc(char *, uint8_t, char **, uint16_t *, char *);
 char *uwsgi_do_rpc(char *, char *, uint8_t, char **, uint16_t *, uint16_t *);
 void uwsgi_rpc_init(void);
@@ -2824,6 +2859,7 @@ void uwsgi_unix_signal(int, void (*)(int));
 char *uwsgi_get_exported_opt(char *);
 
 int uwsgi_signal_add_cron(uint8_t, int, int, int, int, int);
+int uwsgi_cron_task_needs_execution(struct tm *, int, int, int, int, int);
 
 char *uwsgi_get_optname_by_index(int);
 
@@ -3180,7 +3216,9 @@ void uwsgi_opt_add_cron(char *, char *, void *);
 void uwsgi_opt_load_plugin(char *, char *, void *);
 void uwsgi_opt_load_dl(char *, char *, void *);
 void uwsgi_opt_load(char *, char *, void *);
+void uwsgi_opt_safe_fd(char *, char *, void *);
 #ifdef UWSGI_SSL
+void uwsgi_opt_add_legion_cron(char *, char *, void *);
 void uwsgi_opt_sni(char *, char *, void *);
 struct uwsgi_string_list *uwsgi_ssl_add_sni_item(char *, char *, char *, char *, char *);
 #endif
@@ -3704,6 +3742,7 @@ char *uwsgi_strip(char *);
 
 #ifdef UWSGI_SSL
 void uwsgi_opt_legion(char *, char *, void *);
+void uwsgi_opt_legion_mcast(char *, char *, void *);
 struct uwsgi_legion *uwsgi_legion_register(char *, char *, char *, char *, char *);
 void uwsgi_opt_legion_node(char *, char *, void *);
 void uwsgi_legion_register_node(struct uwsgi_legion *, char *);
@@ -3755,6 +3794,7 @@ uint64_t uwsgi_be64(char *);
 int uwsgi_websocket_handshake(struct wsgi_request *, char *, uint16_t, char *, uint16_t);
 
 int uwsgi_response_prepare_headers(struct wsgi_request *, char *, uint16_t);
+int uwsgi_response_prepare_headers_int(struct wsgi_request *, int);
 int uwsgi_response_add_header(struct wsgi_request *, char *, uint16_t, char *, uint16_t);
 int uwsgi_response_commit_headers(struct wsgi_request *);
 int uwsgi_response_sendfile_do(struct wsgi_request *, int, size_t, size_t);
@@ -3778,6 +3818,7 @@ ssize_t uwsgi_sendfile_do(int, int, size_t, size_t);
 int uwsgi_proto_base_fix_headers(struct wsgi_request *);
 int uwsgi_response_add_content_length(struct wsgi_request *, uint64_t);
 int uwsgi_response_add_content_range(struct wsgi_request *, uint64_t, uint64_t, uint64_t);
+int uwsgi_response_add_expires(struct wsgi_request *, uint64_t);
 
 const char *uwsgi_http_status_msg(char *, uint16_t *);
 int uwsgi_stats_dump_vars(struct uwsgi_stats *, struct uwsgi_core *);
@@ -3871,7 +3912,7 @@ struct uwsgi_cache_magic_context {
 	uint16_t cache_len;
 };
 
-char *uwsgi_cache_magic_get(char *, uint16_t, uint64_t *, char *);
+char *uwsgi_cache_magic_get(char *, uint16_t, uint64_t *, uint64_t *, char *);
 int uwsgi_cache_magic_set(char *, uint16_t, char *, uint64_t, uint64_t, uint64_t, char *);
 int uwsgi_cache_magic_del(char *, uint16_t, char *);
 int uwsgi_cache_magic_exists(char *, uint16_t, char *);
@@ -3886,6 +3927,7 @@ int uwsgi_emperor_vassal_start(struct uwsgi_instance *);
 int uwsgi_deflate_init(z_stream *, char *, size_t);
 char *uwsgi_deflate(z_stream *, char *, size_t, size_t *);
 void uwsgi_crc32(uint32_t *, char *, size_t);
+struct uwsgi_buffer *uwsgi_gzip(char *, size_t);
 #endif
 
 char *uwsgi_get_cookie(struct wsgi_request *, char *, uint16_t, uint16_t *);
@@ -3899,6 +3941,21 @@ char *uwsgi_get_mime_type(char *, int, size_t *);
 void config_magic_table_fill(char *, char *[]);
 
 int uwsgi_blob_to_response(struct wsgi_request *, char *, size_t);
+struct uwsgi_cron *uwsgi_cron_add(char *);
+int uwsgi_is_full_http(struct uwsgi_buffer *);
+
+int uwsgi_http_date(time_t t, char *);
+
+int uwsgi_apply_transformations(struct wsgi_request *wsgi_req);
+struct uwsgi_transformation *uwsgi_add_transformation(struct wsgi_request *wsgi_req, int (*func)(struct wsgi_request *, struct uwsgi_buffer *, struct uwsgi_buffer **, void *), void *);
+
+void uwsgi_file_write_do(struct uwsgi_string_list *);
+
+int uwsgi_fd_is_safe(int);
+void uwsgi_add_safe_fd(int);
+
+void uwsgi_ipcsem_clear(void);
+char *uwsgi_str_to_hex(char *, size_t);
 
 void uwsgi_check_emperor(void);
 #ifdef UWSGI_AS_SHARED_LIBRARY
